@@ -11,12 +11,12 @@
 // modify it under the terms of the GNU Lesser General Public
 // License as published by the Free Software Foundation; either
 // version 2.1 of the License, or (at your option) any later version.
-// 
+//
 // This library is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 // Lesser General Public License for more details.
-// 
+//
 // You should have received a copy of the GNU Lesser General Public
 // License along with this library; if not, write to the Free Software
 // Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
@@ -43,11 +43,11 @@
 #endif
 
 /** Parse the XML database/configuration file which lists a collection of AIS
-    resources. 
+    resources.
 
     Static methods are used as callbacks for the SAX parser. They do not
     throw exceptions because exceptions from within callbacks are not
-    reliable or portable (although exceptions themselves are not 100%
+    reliable or portable (and exceptions themselves are not 100%
     portable...). To signal errors, the methods record information in the
     AISParserState object. Once any of the error handlers (aisWarning,
     aisError or aisFatalError) is called, construction of an AISResources
@@ -60,64 +60,65 @@
     operator.
 
     This class should be extended so that the line number is added to error
-    messages. 
+    messages.
 
     @see AISResource */
-class AISDatabaseParser {
+class AISDatabaseParser
+{
 private:
     /** States used by AISParserState. These are the states of the SAX parser
-	state-machine. */ 
+    state-machine. */
     enum ParseState {
-	PARSER_START,
-	PARSER_FINISH,
-	AIS,
-	ENTRY,
-	PRIMARY,
-	ANCILLARY,
-	PARSER_UNKNOWN,
-	PARSER_ERROR
+        PARSER_START,
+        PARSER_FINISH,
+        AIS,
+        ENTRY,
+        PRIMARY,
+        ANCILLARY,
+        PARSER_UNKNOWN,
+        PARSER_ERROR
     };
 
     /** This holds the state information for the SAX parser that is used to
-	intern the XML AIS database. The parser is designed to ignore unknown
-	tags and attributes, so long as the input is well-formed. Note that a
-	pointer to an AISResources object is part of the SAX parser state. As
-	the XML input document is parsed, information is added to that
-	object. Also note that an AISParserState object holds a pointer to
-	the xmlParserCtxt which, in turn, holds a pointer to AISParserState
-	(via its \c userData field). This cirular referencing is done because
-	libxml2's SAX parser invokes the callbacks using just the
-	AISParserState instance but we need the whole xmlParserCtxt for some
-	of the callbacks. 
+    intern the XML AIS database. The parser is designed to ignore unknown
+    tags and attributes, so long as the input is well-formed. Note that a
+    pointer to an AISResources object is part of the SAX parser state. As
+    the XML input document is parsed, information is added to that
+    object. Also note that an AISParserState object holds a pointer to
+    the xmlParserCtxt which, in turn, holds a pointer to AISParserState
+    (via its \c userData field). This cirular referencing is done because
+    libxml2's SAX parser invokes the callbacks using just the
+    AISParserState instance but we need the whole xmlParserCtxt for some
+    of the callbacks. 
 
-	@see aisWarning. */
-    struct AISParserState {
-	ParseState state;	// current state
-	ParseState prev_state;	// previous state
-	int unknown_depth;	// handle recursive unknown tags
- 
-	string error_msg;	// Error message(s), if any.
+    @see aisWarning. */
+    struct AISParserState
+    {
+        ParseState state; // current state
+        ParseState prev_state; // previous state
+        int unknown_depth; // handle recursive unknown tags
 
-	xmlParserCtxtPtr ctxt;	// used for error msg line numbers
-	AISResources *ais; 	// dump info here
+        string error_msg; // Error message(s), if any.
 
-	string primary;		// current entry's primary URL/Regexp
-	bool regexp;		// True if primary is a regexp
+        xmlParserCtxtPtr ctxt; // used for error msg line numbers
+        AISResources *ais;  // dump info here
 
-	ResourceVector rv;	// add ancillary entries to rv
+        string primary;  // current entry's primary URL/Regexp
+        bool regexp;  // True if primary is a regexp
+
+        ResourceVector rv; // add ancillary entries to rv
     };
-    
+
 public:
-    void intern(const string &database, AISResources *ais)
-	throw(AISDatabaseReadFailed);
+    void intern(const string &database, AISResources *ais);
 
     static void aisStartDocument(AISParserState *state);
     static void aisEndDocument(AISParserState *state);
-    static void aisStartElement(AISParserState *state, const char *name, 
-				const char **attrs);
+    static void aisStartElement(AISParserState *state, const char *name,
+                                const char **attrs);
     static void aisEndElement(AISParserState *state, const char *name);
     static xmlEntityPtr aisGetEntity(AISParserState *state,
-				     const xmlChar *name);
+                                     const xmlChar *name);
     static void aisWarning(AISParserState *state, const char *msg, ...);
     static void aisError(AISParserState *state, const char *msg, ...);
     static void aisFatalError(AISParserState *state, const char *msg, ...);
