@@ -11,18 +11,18 @@
 // modify it under the terms of the GNU Lesser General Public
 // License as published by the Free Software Foundation; either
 // version 2.1 of the License, or (at your option) any later version.
-// 
+//
 // This library is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 // Lesser General Public License for more details.
-// 
+//
 // You should have received a copy of the GNU Lesser General Public
 // License along with this library; if not, write to the Free Software
 // Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 //
 // You can contact OPeNDAP, Inc. at PO Box 112, Saunderstown, RI. 02874-0112.
- 
+
 // (c) COPYRIGHT URI/MIT 1994-1999
 // Please read the full copyright statement in the file COPYRIGHT_URI.
 //
@@ -30,7 +30,7 @@
 //      jhrg,jimg       James Gallagher <jgallagher@gso.uri.edu>
 
 // Class for array variables. The dimensions of the array are stored in the
-// list SHAPE. 
+// list SHAPE.
 //
 // jhrg 9/6/94
 
@@ -95,52 +95,54 @@ const int DODS_MAX_ARRAY = DODS_INT_MAX;
     @see Vector
     @see dimension */
 
-class Array: public Vector {
+class Array: public Vector
+{
 public:
     /** Information about a dimension. Each Array has one or more dimensions.
-	For each of an Array's dimensions, a cooresponding instance of this
-	struct holds the natural size, name, constraint information and
-	constrained size.
+     For each of an Array's dimensions, a cooresponding instance of this
+     struct holds the natural size, name, constraint information and
+     constrained size.
 
         @note Instead of using this struct's fileds directly, use Array's
         dimension accessor methods.
 
         @note This sturct is public because its type is used in public
         typedefs. */
-    struct dimension {
-    	int size;		///< The unconstrained dimension size.
-    	string name;    ///< The name of this dimension.
-    	int start;		///< The constriant start index
-    	int stop;		///< The constraint end index
-    	int stride;		///< The constraint stride
-    	int c_size;		///< Size of dimension once constrained
-    	bool selected;	///< \c True if a constraint has been applied to this dimension.
+    struct dimension
+    {
+        int size;  ///< The unconstrained dimension size.
+        string name;    ///< The name of this dimension.
+        int start;  ///< The constriant start index
+        int stop;  ///< The constraint end index
+        int stride;  ///< The constraint stride
+        int c_size;  ///< Size of dimension once constrained
+        bool selected; ///< \c True if a constraint has been applied to this dimension.
     };
 
 private:
-    std::vector<dimension> _shape;	// list of dimensions (i.e., the shape)
-    unsigned int print_array(FILE *out, unsigned int index, 
-			     unsigned int dims, unsigned int shape[]);
+    std::vector<dimension> _shape; // list of dimensions (i.e., the shape)
+    unsigned int print_array(FILE *out, unsigned int index,
+                             unsigned int dims, unsigned int shape[]);
 
     friend class ArrayTest;
-    		     
+
 protected:
     void _duplicate(const Array &a);
     void print_xml_core(FILE *out, string space, bool constrained, string tag);
 
 public:
     /** A constant iterator used to access the various dimensions of an
-	Array. 
+    Array. 
 
         @see dim_begin()
-	@see dim_end() */
+    @see dim_end() */
     typedef std::vector<dimension>::const_iterator Dim_citer ;
     /** An iterator used to access the various dimensions of an
-	Array. Most of the methods that access various properties of a
-	dimension use an instance of Dim_iter.
+    Array. Most of the methods that access various properties of a
+    dimension use an instance of Dim_iter.
 
         @see dim_begin()
-	@see dim_end() */
+    @see dim_end() */
     typedef std::vector<dimension>::iterator Dim_iter ;
 
     Array(const string &n = "", BaseType *v = 0);
@@ -151,7 +153,7 @@ public:
     virtual BaseType *ptr_duplicate();
 
     void add_var(BaseType *v, Part p = nil);
-    
+
     void update_length(int size);
 
     void append_dim(int size, string name = "");
@@ -173,20 +175,22 @@ public:
     unsigned int dimensions(bool constrained = false);
 
     virtual void print_decl(FILE *out, string space = "    ",
-			    bool print_semi = true,
-			    bool constraint_info = false,
-			    bool constrained = false);
+                            bool print_semi = true,
+                            bool constraint_info = false,
+                            bool constrained = false);
 
     virtual void print_xml(FILE *out, string space = "    ",
-			   bool constrained = false);
+                           bool constrained = false);
 
     virtual void print_as_map_xml(FILE *out, string space = "    ",
-				  bool constrained = false);
+                                  bool constrained = false);
 
-    virtual void print_val(FILE *out, string space = "", 
-			   bool print_decl_p = true);
+    virtual void print_val(FILE *out, string space = "",
+                           bool print_decl_p = true);
 
     virtual bool check_semantics(string &msg, bool all = false);
+
+    virtual void dump(ostream &strm) const ;
 };
 
 #endif // _array_h
