@@ -11,18 +11,18 @@
 // modify it under the terms of the GNU Lesser General Public
 // License as published by the Free Software Foundation; either
 // version 2.1 of the License, or (at your option) any later version.
-// 
+//
 // This library is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 // Lesser General Public License for more details.
-// 
+//
 // You should have received a copy of the GNU Lesser General Public
 // License along with this library; if not, write to the Free Software
 // Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 //
 // You can contact OPeNDAP, Inc. at PO Box 112, Saunderstown, RI. 02874-0112.
- 
+
 // (c) COPYRIGHT URI/MIT 1994-1999
 // Please read the full copyright statement in the file COPYRIGHT_URI.
 //
@@ -36,27 +36,29 @@
 
 #include "config.h"
 
-#include <stdio.h>		// for stdin and stdout
+#include <stdio.h>  // for stdin and stdout
 
 #include <sstream>
 #include <string>
 
 // #define DODS_DEBUG
 
-#include "debug.h"
 #include "BaseType.h"
-#include "util.h"
 #include "InternalErr.h"
+
+#include "util.h"
 #include "escaping.h"
 
+#include "debug.h"
+
 using namespace std;
 
 // Protected copy mfunc
 
 /** Perform a deep copy. Copies the values of \e bt into \c *this. Pointers
     are dereferenced and their values are copied into a newly allocated
-    instance. 
-    
+    instance.
+
     @brief Perform a deep copy.
     @param bt The source object. */
 void
@@ -64,15 +66,15 @@ BaseType::_duplicate(const BaseType &bt)
 {
     _name = bt._name;
     _type = bt._type;
-    _read_p = bt._read_p;	// added, reza
-    _send_p = bt._send_p;	// added, reza
+    _read_p = bt._read_p; // added, reza
+    _send_p = bt._send_p; // added, reza
     d_in_selection = bt.d_in_selection;
     _synthesized_p = bt._synthesized_p; // 5/11/2001 jhrg
-    _xdr_coder = bt._xdr_coder;	// just copy this function pointer
+    _xdr_coder = bt._xdr_coder; // just copy this function pointer
 
-    d_parent = bt.d_parent;	// copy pointers 6/4/2001 jhrg
+    d_parent = bt.d_parent; // copy pointers 6/4/2001 jhrg
 
-    d_attr = bt.d_attr;		// Deep copy.
+    d_attr = bt.d_attr;  // Deep copy.
 }
 
 // Public mfuncs
@@ -81,7 +83,7 @@ BaseType::_duplicate(const BaseType &bt)
     an XDR filter.  The BaseType class exists to provide data to
     type classes that inherit from it.  The constructors of those
     classes call the BaseType constructor; it is never called
-    directly. 
+    directly.
 
     Note that the constructor (as well as the copy constructor via
     _duplicate) open/initialize the (XDRS *)s XDRIN and XDROUT to
@@ -98,17 +100,16 @@ BaseType::_duplicate(const BaseType &bt)
     data in this variable to a client DODS process.
     @see Type */
 BaseType::BaseType(const string &n, const Type &t, xdrproc_t xdr)
-    : _name(n), _type(t), _xdr_coder(xdr), _read_p(false), _send_p(false),
-      d_in_selection(false), _synthesized_p(false), d_parent(0)
-{
-} 
+        : _name(n), _type(t), _xdr_coder(xdr), _read_p(false), _send_p(false),
+        d_in_selection(false), _synthesized_p(false), d_parent(0)
+{}
 
 /** @brief The BaseType copy constructor. */
-BaseType::BaseType(const BaseType &copy_from)
+BaseType::BaseType(const BaseType &copy_from) : DapObj()
 {
     _duplicate(copy_from);
 }
-    
+
 BaseType::~BaseType()
 {
     DBG(cerr << "Entering ~BaseType (" << this << ")" << endl);
@@ -119,7 +120,7 @@ BaseType &
 BaseType::operator=(const BaseType &rhs)
 {
     if (this == &rhs)
-	    return *this;
+        return *this;
 
     _duplicate(rhs);
 
@@ -135,31 +136,60 @@ BaseType::toString()
 {
     ostringstream oss;
     oss << "BaseType (" << this << "):" << endl
-	<< "          _name: " << _name << endl
-	<< "          _type: " << type_name() << endl
-	<< "          _read_p: " << _read_p << endl
-	<< "          _send_p: " << _send_p << endl
-	<< "          _synthesized_p: " << _synthesized_p << endl 
-	<< "          d_parent: " << d_parent << endl
-	<< "          d_attr: " << hex << &d_attr << dec << endl;
+    << "          _name: " << _name << endl
+    << "          _type: " << type_name() << endl
+    << "          _read_p: " << _read_p << endl
+    << "          _send_p: " << _send_p << endl
+    << "          _synthesized_p: " << _synthesized_p << endl
+    << "          d_parent: " << d_parent << endl
+    << "          d_attr: " << hex << &d_attr << dec << endl;
 
     return oss.str();
 }
 
-/** @brief Returns the name of the class instance. 
+/** @brief dumps information about this object
+ *
+ * Displays the pointer value of this instance and then displays information
+ * about this base type.
+ *
+ * @param strm C++ i/o stream to dump the information to
+ * @return void
  */
-string 
+void
+BaseType::dump(ostream &strm) const
+{
+    strm << DapIndent::LMarg << "BaseType::dump - ("
+    << (void *)this << ")" << endl ;
+    DapIndent::Indent() ;
+
+    strm << DapIndent::LMarg << "name: " << _name << endl ;
+    strm << DapIndent::LMarg << "type: " << type_name() << endl ;
+    strm << DapIndent::LMarg << "read_p: " << _read_p << endl ;
+    strm << DapIndent::LMarg << "send_p: " << _send_p << endl ;
+    strm << DapIndent::LMarg << "synthesized_p: " << _synthesized_p << endl ;
+    strm << DapIndent::LMarg << "parent: " << (void *)d_parent << endl ;
+    strm << DapIndent::LMarg << "attributes: " << endl ;
+    DapIndent::Indent() ;
+    d_attr.dump(strm) ;
+    DapIndent::UnIndent() ;
+
+    DapIndent::UnIndent() ;
+}
+
+/** @brief Returns the name of the class instance.
+ */
+string
 BaseType::name() const
 {
-    return _name; 
+    return _name;
 }
 
 /** @brief Sets the name of the class instance. */
-void 
+void
 BaseType::set_name(const string &n)
-{ 
+{
     string name = n;
-    _name = www2id(name);	// www2id writes into its param.
+    _name = www2id(name); // www2id writes into its param.
 }
 
 /** @brief Returns the type of the class instance. */
@@ -180,119 +210,121 @@ BaseType::set_type(const Type &t)
 string
 BaseType::type_name() const
 {
-    switch(_type) {
-      case dods_null_c:
-	return string("Null");
-      case dods_byte_c:
-	return string("Byte");
-      case dods_int16_c:
-	return string("Int16");
-      case dods_uint16_c:
-	return string("UInt16");
-      case dods_int32_c:
-	return string("Int32");
-      case dods_uint32_c:
-	return string("UInt32");
-      case dods_float32_c:
-	return string("Float32");
-      case dods_float64_c:
-	return string("Float64");
-      case dods_str_c:
-	return string("String");
-      case dods_url_c:
-	return string("Url");
-      case dods_array_c:
-	return string("Array");
-      case dods_structure_c:
-	return string("Structure");
-      case dods_sequence_c:
-	return string("Sequence");
-      case dods_grid_c:
-	return string("Grid");
-      default:
-	cerr << "BaseType::type_name: Undefined type" << endl;
-	return string("");
+    switch (_type) {
+    case dods_null_c:
+        return string("Null");
+    case dods_byte_c:
+        return string("Byte");
+    case dods_int16_c:
+        return string("Int16");
+    case dods_uint16_c:
+        return string("UInt16");
+    case dods_int32_c:
+        return string("Int32");
+    case dods_uint32_c:
+        return string("UInt32");
+    case dods_float32_c:
+        return string("Float32");
+    case dods_float64_c:
+        return string("Float64");
+    case dods_str_c:
+        return string("String");
+    case dods_url_c:
+        return string("Url");
+    case dods_array_c:
+        return string("Array");
+    case dods_structure_c:
+        return string("Structure");
+    case dods_sequence_c:
+        return string("Sequence");
+    case dods_grid_c:
+        return string("Grid");
+    default:
+        cerr << "BaseType::type_name: Undefined type" << endl;
+        return string("");
     }
 }
 
-/** @brief Returns true if the instance is a simple type variable. */
+/** @brief Returns true if the instance is a numeric. string or URL
+    type variable. */
 bool
 BaseType::is_simple_type()
 {
     switch (type()) {
-      case dods_null_c:
-      case dods_byte_c:
-      case dods_int16_c:
-      case dods_uint16_c:
-      case dods_int32_c:
-      case dods_uint32_c:
-      case dods_float32_c:
-      case dods_float64_c:
-      case dods_str_c:
-      case dods_url_c:
-	return true;
+    case dods_null_c:
+    case dods_byte_c:
+    case dods_int16_c:
+    case dods_uint16_c:
+    case dods_int32_c:
+    case dods_uint32_c:
+    case dods_float32_c:
+    case dods_float64_c:
+    case dods_str_c:
+    case dods_url_c:
+        return true;
 
-      case dods_array_c:
-      case dods_structure_c:
-      case dods_sequence_c:
-      case dods_grid_c:
-	return false;
+    case dods_array_c:
+    case dods_structure_c:
+    case dods_sequence_c:
+    case dods_grid_c:
+        return false;
     }
 
     return false;
 }
 
-/** @brief Returns true if the instance is a vector type variable. */
+/** @brief Returns true if the instance is a vector (i.e., array) type variable. */
 bool
 BaseType::is_vector_type()
 {
     switch (type()) {
-      case dods_null_c:
-      case dods_byte_c:
-      case dods_int16_c:
-      case dods_uint16_c:
-      case dods_int32_c:
-      case dods_uint32_c:
-      case dods_float32_c:
-      case dods_float64_c:
-      case dods_str_c:
-      case dods_url_c:
-	return false;
+    case dods_null_c:
+    case dods_byte_c:
+    case dods_int16_c:
+    case dods_uint16_c:
+    case dods_int32_c:
+    case dods_uint32_c:
+    case dods_float32_c:
+    case dods_float64_c:
+    case dods_str_c:
+    case dods_url_c:
+        return false;
 
-      case dods_array_c:
-	return true;
+    case dods_array_c:
+        return true;
 
-      case dods_structure_c:
-      case dods_sequence_c:
-      case dods_grid_c:
-	return false;
+    case dods_structure_c:
+    case dods_sequence_c:
+    case dods_grid_c:
+        return false;
     }
-    
+
     return false;
 }
 
-/** @brief Returns true if the instance is a constructor type variable. */
+/** @brief Returns true if the instance is a constructor (i.e., Structure,
+    Sequence or Grid) type variable. */
 bool
 BaseType::is_constructor_type()
 {
     switch (type()) {
-      case dods_null_c:
-      case dods_byte_c:
-      case dods_int16_c:
-      case dods_uint16_c:
-      case dods_int32_c:
-      case dods_uint32_c:
-      case dods_float32_c:
-      case dods_float64_c:
-      case dods_str_c:
-      case dods_url_c:
-      case dods_array_c:
-	return false;
+    case dods_null_c:
+    case dods_byte_c:
+    case dods_int16_c:
+    case dods_uint16_c:
+    case dods_int32_c:
+    case dods_uint32_c:
+    case dods_float32_c:
+    case dods_float64_c:
+    case dods_str_c:
+    case dods_url_c:
+    case dods_array_c:
+        return false;
 
-      case dods_structure_c:
-      case dods_sequence_c:
-      case dods_grid_c:
-	return true;
+    case dods_structure_c:
+    case dods_sequence_c:
+    case dods_grid_c:
+        return true;
     }
 
     return false;
@@ -313,7 +345,7 @@ BaseType::is_constructor_type()
     in the subsidiary Structure).  Use the <i>leaves</i> parameter
     to control which kind of counting you desire.
 
-    @brief Count the members of constructor types. 
+    @brief Count the members of constructor types.
     @return Returns 1 for simple
     types.  For compound members, the count depends on the
     <i>leaves</i> argument.
@@ -341,7 +373,7 @@ BaseType::synthesized_p()
 /** Set the synthesized flag. Before setting this flag be sure to set the
     <tt>read_p()</tt> state. Once this flag is set you cannot
     alter the state of the <tt>read_p</tt> flag!
-	
+
     @see synthesized_p() */
 void
 BaseType::set_synthesized_p(bool state)
@@ -370,15 +402,15 @@ BaseType::read_p()
     value(s) of this variable has/have been read. An implementation of the
     read() method would typically use this to set the \c read_p property to
     true.
-    
+
     @note For most of the types the default implementation of this method is
-    fine. However, if you're building a server which must handle data 
+    fine. However, if you're building a server which must handle data
     represented using nested sequences, then you may need to provide a
     specialization of Sequence::set_read_p(). By default Sequence::set_read_()
     recursively sets the \e read_p property for all child variables to
     \e state. For servers where one Sequence reads an outer set of values
     and another reads an inner set, this is cumbersome. In such a case, it is
-    easier to specialize Sequence::set_read_p() so that it does \e not 
+    easier to specialize Sequence::set_read_p() so that it does \e not
     recursively set the \e read_p property for the inner Sequence. Be sure
     to see the documentation for the read() method!
 
@@ -386,7 +418,7 @@ BaseType::read_p()
     synthesized variable is added to a Sequence, the Sequence can iteratively
     reset the \e read_p property without affecting the value of that property
     for the synthesized variable. That's important since a synthesized
-    variable's value is calculated, not read. 
+    variable's value is calculated, not read.
 
     @todo Look at making synthesized variables easier to implement and at
     making them more integrated into the overall CE evaluation process.
@@ -403,8 +435,8 @@ void
 BaseType::set_read_p(bool state)
 {
     if (! _synthesized_p) {
-	DBG(cerr << "Changing read_p state of " << name() << endl);
-	_read_p = state;
+        DBG(cerr << "Changing read_p state of " << name() << endl);
+        _read_p = state;
     }
 }
 
@@ -431,18 +463,18 @@ BaseType::send_p()
 
     @param state The logical state to set the <tt>send_p</tt> flag.
 */
-void 
+void
 BaseType::set_send_p(bool state)
 {
-    DBG(cerr << "Calling BaseType::set_send_p() for: " << this->name() 
-	<< endl);
+    DBG(cerr << "Calling BaseType::set_send_p() for: " << this->name()
+        << endl);
     _send_p = state;
 }
 
 
 /** Get this variable's AttrTable. It's generally a bad idea to return a
     reference to a contained object, but in this case it seems that building
-    an interface inside BaseType is overkill. 
+    an interface inside BaseType is overkill.
 
     Use the AttrTable methods to manipulate the table. */
 AttrTable &
@@ -461,12 +493,16 @@ BaseType::set_attr_table(const AttrTable &at)
 
 /** Does this variable appear in either the selection part or as a function
     argument in the current constrain expression. If this property is set
-    (true) then implementation of the read() method should read this
+    (true) then implementations of the read() method should read this
     variable.
 
+    @note This method does not check, nor does it know about the semantics of,
+    string arguments passed to functions. Those functions might include
+    variable names in strings; they are responsible for reading those variables.
+    See the grid (func_grid_select()) for an example.
     @see BaseType::read()
     @brief Is this variable part of the current selection? */
-bool 
+bool
 BaseType::is_in_selection()
 {
     return d_in_selection;
@@ -479,7 +515,8 @@ BaseType::is_in_selection()
     this variable.
 
     @param state Set the \e in_selection property to this state.
-    @see BaseType::read() */
+    @see BaseType::read()
+    @see BaseType::is_in_selection() for more information. */
 void
 BaseType::set_in_selection(bool state)
 {
@@ -494,11 +531,11 @@ BaseType::set_in_selection(bool state)
     @exception InternalErr thrown if called with anything other than a
     Constructor or Vector. */
 void
-BaseType::set_parent(BaseType *parent) throw(InternalErr)
+BaseType::set_parent(BaseType *parent)
 {
     if (!dynamic_cast<Constructor *>(parent)
-	&& !dynamic_cast<Vector *>(parent))
-	throw InternalErr("Call to set_parent with incorrect variable type.");
+        && !dynamic_cast<Vector *>(parent))
+        throw InternalErr("Call to set_parent with incorrect variable type.");
 
     d_parent = parent;
 }
@@ -553,42 +590,44 @@ BaseType::var(const string &, btp_stack &)
     classes. For constructors with more than one variable, the variables
     appear in the same order in which they were added (i.e., the order in
     which add_var() was called). Since this method is only for use by Vectors
-    and Constructors, the BaseType implementation does nothing.
+    and Constructors, the BaseType implementation throws InternalErr.
 
-    @note Implementation of this method in Structure, Sequence, et c., first
-    copy \e bt and then insert the copy. If \e bt is itself a constructor
+    @note For the implementation of this method in Structure, Sequence, et c.,
+    first copy \e bt and then insert the copy. If \e bt is itself a constructor
     type you must either use the var() method to get a pointer to the actual
     instance added to \c *this or you must first add all of <em>bt</em>'s
-    children to it before adding it to \c *this. The implementations use
+    children to it before adding it to \c *this. The implementations should use
     _duplicate() to perform a deep copy of \e bt.
 
     @brief Add a variable.
 
-    @todo The BaseType implementation of this method should throw an
-    InternalErr.
-
     @todo We should get rid of the Part parameter and adopt the convention
     that the first variable is the Array and all subsequent ones are Maps
     (when dealing with a Grid, the only time Part matters). This would enable
     several methods to migrate from Structure, Sequence and Grid to
-    Constructor. 
+    Constructor.
 
     @param bt The variable to be added to this instance. The caller of this
     method <i>must</i> free memory it allocates for <tt>v</tt>. This method
     will make a deep copy of the object pointed to by <tt>v</tt>.
     @param part The part of the constructor data to be modified. Only
     meaningful for Grid variables.
-    
+
     @see Part */
 void
 BaseType::add_var(BaseType *, Part)
 {
+    throw InternalErr(__FILE__, __LINE__, "BaseType::add_var unimplemented");
 }
 
 /** This method should be implemented for each of the data type classes (Byte,
     ..., Grid) when using the DAP class library to build a server. This
     method is only for DAP servers. The library provides a default
-    definition here which throws an InternalErr exception.
+    definition here which throws an InternalErr exception \e unless the read_p
+    property has been set. In that case it returns false, indicating that all
+    the data have been read. The latter case can happen when building a
+    constant value that needs to be passed to a function. The variable/constant
+    is loaded with a value when it is created.
 
     When implementing a new DAP server, the Byte, ..., Grid data type classes
     are usually specialized. In each of those specializations read() should
@@ -598,23 +637,23 @@ BaseType::add_var(BaseType *, Part)
 
     <ul>
     <li> read() should throw Error if it encounters an error. The message
-	  should be verbose enough to be understood by someone running a
-	  client on a different machine.</li>
+   should be verbose enough to be understood by someone running a
+   client on a different machine.</li>
     <li> The value(s) should be read if and only if either send_p() or
           is_in_selection() return true. If neither of these return true, the
-	  value(s) should not be read. This is important when writing read()
-	  for a Constructor type such as Grid where a client may ask for only
-	  the map vectors (and thus reading the much larger Array part is not
-	  needed).</li>
+   value(s) should not be read. This is important when writing read()
+   for a Constructor type such as Grid where a client may ask for only
+   the map vectors (and thus reading the much larger Array part is not
+   needed).</li>
     <li> For each specialization of read(), the method should first test
           the value of the \c read_p property (using the read_p() method)
-          and read values only if the value of read_p() is false. Once the 
+          and read values only if the value of read_p() is false. Once the
           read() method reads data and stores it in the instance, it must
           set the value of the \c read_p property to true using set_read_p().
           If your read() methods fail to do this data may not serialize
           correctly.</li>
     <li> The Array::read() and Grid::read() methods should take into account
-	  any restrictions on Array sizes.</li>
+   any restrictions on Array sizes.</li>
     <li> If you are writing Sequence::read(), be sure to check the
           documentation for Sequence::read_row() and Sequence::serialize()
           so you understand how Sequence::read() is being called.</li>
@@ -627,7 +666,7 @@ BaseType::add_var(BaseType *, Part)
           counter must be reset (use Sequence::reset_row_counter()). Typically
           the correct time to set \c unsent_data to false and reset the row
           counter is the time when Sequence::read() return false indicating
-          that all the data for the Sequence have been read. Failure to 
+          that all the data for the Sequence have been read. Failure to
           handle these tasks will break serialization of nested Sequences. Note
           that when Sequence::read() returns with a result of true (indicting
           there is more data to send, the value of the \c unsent_data property
@@ -637,23 +676,26 @@ BaseType::add_var(BaseType *, Part)
           about subclassing set_read_p().</li>
     </ul>
 
-    @brief Read data into a local buffer. 
+    @brief Read data into a local buffer.
 
-    @return The return value of this method for all types except Sequence 
+    @return The return value of this method for all types except Sequence
     should always be false. Sequences should return true to indicate more
     values remain in the Sequence, false to indicate no more values remain.
     (see Sequence::serialize() and Sequence::read_row()).
 
     @param dataset A string naming the dataset from which the data is to
     be read. The meaning of this string will vary among different types of
-    data sources. It \e may be the name of a data file or an identifier 
+    data sources. It \e may be the name of a data file or an identifier
     used to read data from a relational database.
 
     @see BaseType
     @see Sequence  */
-bool 
+bool
 BaseType::read(const string &)
 {
+    if (_read_p)
+        return false;
+
     throw InternalErr("Unimplemented BaseType::read() method called.");
 }
 
@@ -668,7 +710,7 @@ BaseType::read(const string &)
     types.  The simple data types (Int, Float, and so on), are
     translated directly.
 
-    @brief Returns a function used to encode elements of an array. 
+    @brief Returns a function used to encode elements of an array.
     @return A C function used to encode data in the XDR format.
 */
 xdrproc_t
@@ -711,7 +753,7 @@ BaseType::xdr_coder()
     constraint information is to be printed with the declaration.
     If the value of this parameter is TRUE, <tt>print_decl()</tt> prints
     the value of the variable's <tt>send_p()</tt> flag after the
-    declaration. 
+    declaration.
     @param constrained If this boolean value is TRUE, the variable's
     declaration is only printed if is the <tt>send_p()</tt> flag is TRUE.
     If a constraint expression is in place, and this variable is not
@@ -720,27 +762,27 @@ BaseType::xdr_coder()
     @see DDS
     @see DDS::CE
 */
-void 
-BaseType::print_decl(FILE *out, string space, bool print_semi, 
-		     bool constraint_info, bool constrained)
+void
+BaseType::print_decl(FILE *out, string space, bool print_semi,
+                     bool constraint_info, bool constrained)
 {
     // if printing the constrained declaration, exit if this variable was not
-    // selected. 
+    // selected.
     if (constrained && !send_p())
-	return;
+        return;
 
-    fprintf( out, "%s%s %s", space.c_str(), type_name().c_str(),
-	     id2www(_name).c_str() ) ;
+    fprintf(out, "%s%s %s", space.c_str(), type_name().c_str(),
+            id2www(_name).c_str()) ;
 
     if (constraint_info) {
-	if (send_p())
-	    fprintf( stdout, ": Send True" ) ;
-	else
-	    fprintf( stdout, ": Send False" ) ;
+        if (send_p())
+            fprintf(stdout, ": Send True") ;
+        else
+            fprintf(stdout, ": Send False") ;
     }
 
     if (print_semi)
-	fprintf( out, ";\n" ) ;
+        fprintf(out, ";\n") ;
 }
 
 /** Write the XML representation of this variable. This method is used to
@@ -753,20 +795,20 @@ void
 BaseType::print_xml(FILE *out, string space, bool constrained)
 {
     if (constrained && !send_p())
-	return;
+        return;
 
     fprintf(out, "%s<%s", space.c_str(), type_name().c_str());
     if (!_name.empty())
-	fprintf(out, " name=\"%s\"", id2xml(_name).c_str());
+        fprintf(out, " name=\"%s\"", id2xml(_name).c_str());
 
     if (get_attr_table().get_size() > 0) {
-	fprintf(out, ">\n");	// close the varaible's tag
-	get_attr_table().print_xml(out, space + "    ", constrained);
-	// After attributes, print closing tag
-	fprintf(out, "%s</%s>\n", space.c_str(), type_name().c_str());
+        fprintf(out, ">\n"); // close the varaible's tag
+        get_attr_table().print_xml(out, space + "    ", constrained);
+        // After attributes, print closing tag
+        fprintf(out, "%s</%s>\n", space.c_str(), type_name().c_str());
     }
     else {
-	fprintf(out, "/>\n");	// no attributes; just close tag.
+        fprintf(out, "/>\n"); // no attributes; just close tag.
     }
 }
 
@@ -800,22 +842,22 @@ BaseType::print_xml(FILE *out, string space, bool constrained)
     type.
     @return Returns FALSE when the current state violates some
     aspect of the type semantics, TRUE otherwise.
-  
+
     @param msg A returned string, containing a message indicating
     the source of any problem.
     @param all For complex constructor types (Grid,
     Sequence, Structure), this flag indicates whether to check the
     sematics of the member variables, too.
 
-    @see DDS::check_semantics 
+    @see DDS::check_semantics
 */
 bool
 BaseType::check_semantics(string &msg, bool)
 {
     bool sem = (_type != dods_null_c && _name.length());
 
-    if (!sem) 
-	msg = "Every variable must have both a name and a type\n";
+    if (!sem)
+        msg = "Every variable must have both a name and a type\n";
 
     return sem;
 }
@@ -852,11 +894,11 @@ BaseType::check_semantics(string &msg, bool)
     @param op An integer index indicating which relational operator
     is implied. Choose one from the following: <tt>EQUAL</tt>,
     <tt>NOT_EQUAL</tt>, <tt>GREATER</tt>, <tt>GREATER_EQL</tt>,
-    <tt>LESS</tt>, <tt>LESS_EQL</tt>, and <tt>REGEXP</tt>. 
+    <tt>LESS</tt>, <tt>LESS_EQL</tt>, and <tt>REGEXP</tt>.
     @param dataset The name of the dataset from which the instance's
     data has come (or is to come).
     @return The boolean value of the comparison. */
-bool 
+bool
 BaseType::ops(BaseType *, int, const string &)
 {
     // Even though ops is a public method, it can never be called because
