@@ -11,23 +11,23 @@
 // modify it under the terms of the GNU Lesser General Public
 // License as published by the Free Software Foundation; either
 // version 2.1 of the License, or (at your option) any later version.
-// 
+//
 // This library is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 // Lesser General Public License for more details.
-// 
+//
 // You should have received a copy of the GNU Lesser General Public
 // License along with this library; if not, write to the Free Software
 // Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 //
 // You can contact OPeNDAP, Inc. at PO Box 112, Saunderstown, RI. 02874-0112.
- 
+
 // (c) COPRIGHT URI/MIT 1995-1999
 // Please first read the full copyright statement in the file COPYRIGHT_URI.
 //
 // Authors:
-//	jhrg,jimg	James Gallagher <jgallagher@gso.uri.edu>
+// jhrg,jimg James Gallagher <jgallagher@gso.uri.edu>
 
 // Interface for the CE Clause class.
 
@@ -62,7 +62,7 @@
     arguments in the clause to return a boolean value.  For example,
     consider a scalar A and a list L.  The clause <tt>find(A,L)</tt> might
     return TRUE if A is a member of L (if the <tt>find()</tt> function is
-    defined). 
+    defined).
 
     <li> A clause that returns a pointer to a DAP BaseType value.
     This is a clause that evaluates to some data value (be it scalar
@@ -84,7 +84,8 @@
 
     @brief Holds a fragment of a constraint expression.
     @see DDS::parse_constraint */
-struct Clause {
+struct Clause
+{
 
 private:
     /** The relational operator, if any. */
@@ -92,12 +93,19 @@ private:
     /** A pointer to a valid boolean function. */
     bool_func _b_func;
     /** A pointer to a valid function that returns a pointer to a
-	BaseType. */
+    BaseType. */
     btp_func _bt_func;
 
-    int _argc;			// arg count
-    rvalue *_arg1;		// only for operator
-    rvalue_list *_args;		// vector arg
+    int _argc;   // arg count
+    rvalue *_arg1;  // only for operator
+    rvalue_list *_args;  // vector arg
+
+    Clause(const Clause &)
+    {}
+    Clause &operator=(const Clause &)
+    {
+        throw InternalErr(__FILE__, __LINE__, "Unimplemented method");
+    }
 
 public:
     Clause(const int oper, rvalue *a1, rvalue_list *rv);
@@ -107,15 +115,15 @@ public:
 
     virtual ~Clause();
 
-  bool OK();
+    bool OK();
 
-  bool boolean_clause();
+    bool boolean_clause();
 
-  bool value_clause();
+    bool value_clause();
 
-  bool value(const string &dataset, DDS &dds);
+    bool value(const string &dataset, DDS &dds);
 
-  bool value(const string &dataset, DDS &dds, BaseType **value);
+    bool value(const string &dataset, DDS &dds, BaseType **value);
 };
 
 #endif // _clause_h
