@@ -11,18 +11,18 @@
 // modify it under the terms of the GNU Lesser General Public
 // License as published by the Free Software Foundation; either
 // version 2.1 of the License, or (at your option) any later version.
-// 
+//
 // This library is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 // Lesser General Public License for more details.
-// 
+//
 // You should have received a copy of the GNU Lesser General Public
 // License along with this library; if not, write to the Free Software
 // Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 //
 // You can contact OPeNDAP, Inc. at PO Box 112, Saunderstown, RI. 02874-0112.
- 
+
 #ifndef _constructor_h
 #define _constructor_h 1
 
@@ -35,14 +35,18 @@
 #endif
 
 /** Common methods for all constructor types. */
-class Constructor: public BaseType {
+class Constructor: public BaseType
+{
 private:
-    Constructor();		// No default ctor.
+    Constructor();  // No default ctor.
+    BaseType *find_hdf4_dimension_attribute_home(AttrTable::entry *source);
 
 protected:
     std::vector<BaseType *> _vars;
 
     void _duplicate(const Constructor &s);
+    virtual AttrTable *find_matching_container(AttrTable::entry *source,
+            BaseType **dest_variable);
 
 public:
     typedef std::vector<BaseType *>::const_iterator Vars_citer ;
@@ -55,6 +59,8 @@ public:
     virtual ~Constructor();
 
     Constructor &operator=(const Constructor &rhs);
+    virtual void transfer_attributes(AttrTable::entry *entry);
+
     Vars_iter var_begin();
     Vars_iter var_end();
     Vars_riter var_rbegin();
@@ -65,12 +71,14 @@ public:
     virtual bool is_linear();
 
     virtual void print_decl(FILE *out, string space = "    ",
-			    bool print_semi = true,
-			    bool constraint_info = false,
-			    bool constrained = false);
+                            bool print_semi = true,
+                            bool constraint_info = false,
+                            bool constrained = false);
 
-    virtual void print_xml(FILE *out, string space = "    ", 
-			   bool constrained =false);
+    virtual void print_xml(FILE *out, string space = "    ",
+                           bool constrained = false);
+
+    virtual void dump(ostream &strm) const ;
 };
 
-#endif // _constructor_h 
+#endif // _constructor_h
