@@ -11,18 +11,18 @@
 // modify it under the terms of the GNU Lesser General Public
 // License as published by the Free Software Foundation; either
 // version 2.1 of the License, or (at your option) any later version.
-// 
+//
 // This library is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 // Lesser General Public License for more details.
-// 
+//
 // You should have received a copy of the GNU Lesser General Public
 // License along with this library; if not, write to the Free Software
 // Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 //
 // You can contact OPeNDAP, Inc. at PO Box 112, Saunderstown, RI. 02874-0112.
- 
+
 // (c) COPYRIGHT URI/MIT 1999,2000
 // Please read the full copyright statement in the file COPYRIGHT_URI.
 //
@@ -37,7 +37,7 @@
 #define _error_h
 
 
-#include <stdio.h>		// For FILE *
+#include <stdio.h>  // For FILE *
 
 #include <iostream>
 #include <string>
@@ -68,7 +68,7 @@ typedef int ErrorCode; //using standard errno+netCDF error codes from server
 /** @deprecated Use of the error-correction program feature is deprecated. */
 enum ProgramType {
     undefined_prog_type = -1,
-    no_program,			// if there is no program
+    no_program,   // if there is no program
     java,
     tcl
 };
@@ -92,12 +92,13 @@ enum ProgramType {
     @brief A class for error processing.
     @author jhrg */
 
-class Error {
+class Error
+{
 protected:
     ErrorCode _error_code;
     string _error_message;
-    ProgramType _program_type;	///< deprecated; don't use
-    char *_program;		///< deprecated; don't use
+    ProgramType _program_type; ///< deprecated; don't use
+    char *_program;  ///< deprecated; don't use
 
 public:
     /// Create an instance with an error code and a message
@@ -132,7 +133,7 @@ public:
     void set_error_code(ErrorCode ec = undefined_error);
     /// Set the error message.
     void set_error_message(string msg = "");
-#if 0   
+#if 0
     /// Deprecated
     ProgramType get_program_type() const;
     /// Deprecated
