@@ -11,18 +11,18 @@
 // modify it under the terms of the GNU Lesser General Public
 // License as published by the Free Software Foundation; either
 // version 2.1 of the License, or (at your option) any later version.
-// 
+//
 // This library is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 // Lesser General Public License for more details.
-// 
+//
 // You should have received a copy of the GNU Lesser General Public
 // License along with this library; if not, write to the Free Software
 // Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 //
 // You can contact OPeNDAP, Inc. at PO Box 112, Saunderstown, RI. 02874-0112.
- 
+
 // (c) COPYRIGHT URI/MIT 1994-1999
 // Please read the full copyright statement in the file COPYRIGHT_URI.
 //
@@ -48,10 +48,18 @@
 #include "BaseType.h"
 #endif
 
+#ifndef _array_h
+#include "Array.h"
+#endif
+
 #ifndef _constructor_h
 #include "Constructor.h"
 #endif
 
+#ifndef constraint_evaluator_h
+#include "ConstraintEvaluator.h"
+#endif
+
 /** The Grid data type is a collection of an Array and a set of ``Map''
     vectors.  The Map vectors are one-dimensional arrays corresponding
     to each dimension of the central Array.  Using this scheme, a Grid
@@ -70,7 +78,7 @@
     To locate this Array in the real world, we could note the location
     of one corner of the grid, and the grid spacing.  This would allow
     us to calculate the location of any of the other points of the
-    Array. 
+    Array.
 
     This approach will not work, however, unless the grid spacing is
     precisely regular.  If the distance between Row 1 and Row 2 is not
@@ -89,12 +97,12 @@
 
     The real location of the point in the first row and column of the
     array is now exactly fixed at (0,0), and the point in the last row
-    and last column is at (8,27). 
+    and last column is at (8,27).
 
     The Grid data type has two parts: an Array, and a singly-linked
     list of Map vectors to describe the Array.  The access functions
     for this class include a function to return the Array
-    (<tt>array_var()</tt>), and a set of functions for cycling through the
+    (<tt>get_array()</tt>), and a set of functions for cycling through the
     list of Map vectors.
 
     @todo Move, in some sense, the _map_vars up to Constructor. Look at using
@@ -107,11 +115,12 @@
     @see Array
     */
 
-class Grid: public Constructor {
+class Grid: public Constructor
+{
 private:
     BaseType *_array_var;
     std::vector<BaseType *> _map_vars;
-    
+
     void _duplicate(const Grid &s);
 
 public:
@@ -122,8 +131,9 @@ public:
 
     typedef std::vector<BaseType *>::const_iterator Map_citer ;
     typedef std::vector<BaseType *>::iterator Map_iter ;
+    typedef std::vector<BaseType *>::reverse_iterator Map_riter ;
+
 
-    
     Grid &operator=(const Grid &rhs);
     virtual BaseType *ptr_duplicate();
 
@@ -134,13 +144,14 @@ public:
     virtual void set_in_selection(bool state);
 
     virtual BaseType *var(const string &n, bool exact = true,
-			  btp_stack *s = 0);
+                          btp_stack *s = 0);
 
     virtual BaseType *var(const string &n, btp_stack &s);
 
     virtual void add_var(BaseType *bt, Part part);
 
     BaseType *array_var();
+    Array *get_array();
 
     virtual unsigned int width();
 
@@ -148,8 +159,10 @@ public:
 
     virtual bool projection_yields_grid();
 
-    virtual bool serialize(const string &dataset, DDS &dds, XDR *sink,
-			   bool ce_eval = true);
+    virtual void clear_constraint();
+
+    virtual bool serialize(const string &dataset, ConstraintEvaluator &eval,
+                           DDS &dds, XDR *sink, bool ce_eval = true);
     virtual bool deserialize(XDR *source, DDS *dds, bool reuse = false);
 
     virtual unsigned int val2buf(void *buf, bool reuse = false);
@@ -157,25 +170,25 @@ public:
     virtual unsigned int buf2val(void **val);
 
     virtual void print_decl(FILE *out, string space = "    ",
-			    bool print_semi = true,
-			    bool constraint_info = false,
-			    bool constrained = false);
+                            bool print_semi = true,
+                            bool constraint_info = false,
+                            bool constrained = false);
 
-    virtual void print_xml(FILE *out, string space = "    ", 
-			   bool constrained =false);
+    virtual void print_xml(FILE *out, string space = "    ",
+                           bool constrained = false);
 
-     virtual void print_val(FILE *out, string space = "",
-			   bool print_decl_p = true);
+    virtual void print_val(FILE *out, string space = "",
+                           bool print_decl_p = true);
+
+    virtual bool check_semantics(string &msg, bool all = false);
 
-   virtual bool check_semantics(string &msg, bool all = false);
-    /** Returns an iterator referencing the first Map vector. */
     Map_iter map_begin() ;
-
-    /** Returns an iterator referencing the end of the list of Map vectors.
-        It does not reference the last Map vector */
     Map_iter map_end() ;
-
+    Map_riter map_rbegin() ;
+    Map_riter map_rend() ;
     Map_iter get_map_iter(int i);
+
+    virtual void dump(ostream &strm) const ;
 };
 
 #endif // _grid_h
