@@ -11,25 +11,25 @@
 // modify it under the terms of the GNU Lesser General Public
 // License as published by the Free Software Foundation; either
 // version 2.1 of the License, or (at your option) any later version.
-// 
+//
 // This library is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 // Lesser General Public License for more details.
-// 
+//
 // You should have received a copy of the GNU Lesser General Public
 // License along with this library; if not, write to the Free Software
 // Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 //
 // You can contact OPeNDAP, Inc. at PO Box 112, Saunderstown, RI. 02874-0112.
- 
+
 // (c) COPYRIGHT URI/MIT 1997,1999
 // Please read the full copyright statement in the file COPYRIGHT_URI.
 //
 // Authors:
 //      jhrg,jimg       James Gallagher <jgallagher@gso.uri.edu>
 
-// Interface for Int16 type. 
+// Interface for Int16 type.
 //
 // jhrg 9/7/94
 
@@ -45,14 +45,19 @@
 #include "BaseType.h"
 #endif
 
+#ifndef constraint_evaluator_h
+#include "ConstraintEvaluator.h"
+#endif
+
 /** @brief Holds a 16-bit signed integer value. */
 
-class Int16: public BaseType {
-    /** This class allows Byte, ..., Float64 access to <tt>_buf</tt> to 
-	simplify and speed up the relational operators.
+class Int16: public BaseType
+{
+    /** This class allows Byte, ..., Float64 access to <tt>_buf</tt> to
+    simplify and speed up the relational operators.
 
-	NB: According to Stroustrup it does not matter where (public, private
-	or protected) friend classes are declared. */
+    NB: According to Stroustrup it does not matter where (public, private
+    or protected) friend classes are declared. */
     friend class Byte;
     friend class UInt16;
     friend class Int32;
@@ -65,27 +70,33 @@ protected:
 
 public:
     Int16(const string &n = "");
-    virtual ~Int16() {}
+    virtual ~Int16()
+    {}
 
     Int16(const Int16 &copy_from);
 
     Int16 &operator=(const Int16 &rhs);
 
     virtual BaseType *ptr_duplicate();
-    
+
     virtual unsigned int width();
 
-    virtual bool serialize(const string &dataset, DDS &dds, XDR *sink,
-			   bool ce_eval = true);
+    virtual bool serialize(const string &dataset, ConstraintEvaluator &eval,
+                           DDS &dds, XDR *sink, bool ce_eval = true);
     virtual bool deserialize(XDR *source, DDS *dds, bool reuse = false);
 
     virtual unsigned int val2buf(void *val, bool reuse = false);
     virtual unsigned int buf2val(void **val);
 
+    virtual dods_int16 value() const;
+    virtual bool set_value(dods_int16 val);
+
     virtual void print_val(FILE *out, string space = "",
-			   bool print_decl_p = true);
+                           bool print_decl_p = true);
 
     virtual bool ops(BaseType *b, int op, const string &dataset);
+
+    virtual void dump(ostream &strm) const ;
 };
 
 #endif // _int16_h
