@@ -11,18 +11,18 @@
 // modify it under the terms of the GNU Lesser General Public
 // License as published by the Free Software Foundation; either
 // version 2.1 of the License, or (at your option) any later version.
-// 
+//
 // This library is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 // Lesser General Public License for more details.
-// 
+//
 // You should have received a copy of the GNU Lesser General Public
 // License along with this library; if not, write to the Free Software
 // Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 //
 // You can contact OPeNDAP, Inc. at PO Box 112, Saunderstown, RI. 02874-0112.
- 
+
 // (c) COPYRIGHT URI/MIT 1994-1999
 // Please read the full copyright statement in the file COPYRIGHT_URI.
 //
@@ -36,7 +36,9 @@
 
 #include "config.h"
 
-static char rcsid[] not_used = {"$Id: PassiveStr.cc 13269 2006-02-24 18:54:20Z jimg $"};
+static char rcsid[] not_used =
+    {"$Id: PassiveStr.cc 16088 2007-03-28 21:42:19Z jimg $"
+    };
 
 #include "PassiveStr.h"
 #include "debug.h"
@@ -47,20 +49,18 @@ using std::endl;
 
 /** The PassiveStr constructor requires only the name of the variable
     to be created.  The name may be omitted, which will create a
-    nameless variable.  This may be adequate for some applications. 
-      
+    nameless variable.  This may be adequate for some applications.
+
     @param n A string containing the name of the variable to be
-    created. 
+    created.
 
 */
 PassiveStr::PassiveStr(const string &n) : Str(n)
-{
-}
+{}
 
 PassiveStr::PassiveStr(const PassiveStr &copy_from) : Str(copy_from)
-{
-}
-    
+{}
+
 BaseType *
 PassiveStr::ptr_duplicate()
 {
@@ -76,7 +76,7 @@ PassiveStr &
 PassiveStr::operator=(const PassiveStr &rhs)
 {
     if (this == &rhs)
-	return *this;
+        return *this;
 
     // Call BaseType::operator=.
     dynamic_cast<BaseType &>(*this) = rhs;
@@ -87,20 +87,19 @@ PassiveStr::operator=(const PassiveStr &rhs)
 }
 
 bool
-PassiveStr::read( const string & )
+PassiveStr::read(const string &)
 {
-    if( !read_p() )
-    {
-	val2buf( &_buf ) ;
-	set_read_p( true ) ;
+    if (!read_p()) {
+        val2buf(&_buf) ;
+        set_read_p(true) ;
     }
 
     return true ;
 }
 
 bool
-PassiveStr::set_value( const string &val )
+PassiveStr::set_value(const string &val)
 {
     _buf = val ;
-    return read( "" ) ;
+    return read("") ;
 }
