@@ -11,18 +11,18 @@
 // modify it under the terms of the GNU Lesser General Public
 // License as published by the Free Software Foundation; either
 // version 2.1 of the License, or (at your option) any later version.
-// 
+//
 // This library is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 // Lesser General Public License for more details.
-// 
+//
 // You should have received a copy of the GNU Lesser General Public
 // License along with this library; if not, write to the Free Software
 // Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 //
 // You can contact OPeNDAP, Inc. at PO Box 112, Saunderstown, RI. 02874-0112.
- 
+
 #ifndef _signal_handler_registered_err_h
 #define _signal_handler_registered_err_h 1
 
@@ -38,17 +38,22 @@
 
     @author jhrg */
 
-class SignalHandlerRegisteredErr: public Error {
+class SignalHandlerRegisteredErr: public Error
+{
 
 public:
-    SignalHandlerRegisteredErr(const string &msg) : Error() {
-	_error_code=unknown_error;
-	_error_message="";
-	_error_message+="A caching error was encounterd:\n";
-	_error_message+=msg+"\n";
+    SignalHandlerRegisteredErr(const string &msg) : Error()
+    {
+        _error_code = unknown_error;
+        _error_message = "";
+        _error_message += "A caching error was encounterd:\n";
+        _error_message += msg + "\n";
     }
 
-    SignalHandlerRegisteredErr() : Error() { _error_code = unknown_error; }
+    SignalHandlerRegisteredErr() : Error()
+    {
+        _error_code = unknown_error;
+    }
 };
 
 #endif // _signal_handler_registered_err_h
