@@ -6,18 +6,18 @@
 
 // Copyright (c) 2002,2003 OPeNDAP, Inc.
 // Author: James Gallagher <jgallagher@opendap.org>
-//         Reza Nekovei <rnekovei@intcomm.net> 
+//         Reza Nekovei <rnekovei@intcomm.net>
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Lesser General Public
 // License as published by the Free Software Foundation; either
 // version 2.1 of the License, or (at your option) any later version.
-// 
+//
 // This library is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 // Lesser General Public License for more details.
-// 
+//
 // You should have received a copy of the GNU Lesser General Public
 // License along with this library; if not, write to the Free Software
 // Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
@@ -33,11 +33,13 @@
 
 // A few useful routines which are used in CGI programs.
 //
-// ReZa 9/30/94 
+// ReZa 9/30/94
 
 #include "config.h"
 
-static char rcsid[] not_used = {"$Id: cgi_util.cc 13266 2006-02-23 23:33:47Z jimg $"};
+static char rcsid[] not_used =
+    {"$Id: cgi_util.cc 16088 2007-03-28 21:42:19Z jimg $"
+    };
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -69,22 +71,22 @@ static char rcsid[] not_used = {"$Id: cgi_util.cc 13266 2006-02-23 23:33:47Z jim
 #include <string>
 
 #include "cgi_util.h"
-#include "util.h"		// This supplies flush_stream for WIN32.
+#include "util.h"  // This supplies flush_stream for WIN32.
 #include "debug.h"
 
 
 #ifdef WIN32
-#define FILE_DELIMITER	'\\'
+#define FILE_DELIMITER '\\'
 #else  //  default to unix
 #define FILE_DELIMITER '/'
 #endif
 
 // ...not using a const string here to avoid global objects. jhrg 12/23/05
-#define CRLF "\r\n"             // Change here and in expr-test.cc. 
+#define CRLF "\r\n"             // Change here and in expr-test.cc.
 
 using namespace std;
 
-static const int TimLen = 26;	// length of string from asctime()
+static const int TimLen = 26; // length of string from asctime()
 static const int CLUMP_SIZE = 1024; // size of clumps to new in fmakeword()
 
 /** This sends a formatted block of text to the client, containing
@@ -96,84 +98,32 @@ static const int CLUMP_SIZE = 1024; // size of clumps to new in fmakeword()
 
     @brief Send a version number.
     @param script_ver The version of the filter script executing this
-    function. 
+    function.
     @param dataset_ver The version of the dataset.
     @return TRUE for success. Always returns true.
 */
 bool
 do_version(const string &script_ver, const string &dataset_ver)
 {
-    fprintf( stdout, "HTTP/1.0 200 OK%s", CRLF ) ;
-    fprintf( stdout, "XDODS-Server: %s%s", DVR, CRLF ) ;
-    fprintf( stdout, "XOPeNDAP-Server: %s%s", DVR, CRLF ) ;
-    fprintf( stdout, "XDAP: %s%s", DAP_PROTOCOL_VERSION, CRLF ) ;
-    fprintf( stdout, "Content-Type: text/plain%s", CRLF ) ;
-    fprintf( stdout, CRLF ) ;
-    
-    fprintf( stdout, "Core software version: %s%s", DVR, CRLF ) ;
+    fprintf(stdout, "HTTP/1.0 200 OK%s", CRLF) ;
+    fprintf(stdout, "XDODS-Server: %s%s", DVR, CRLF) ;
+    fprintf(stdout, "XOPeNDAP-Server: %s%s", DVR, CRLF) ;
+    fprintf(stdout, "XDAP: %s%s", DAP_PROTOCOL_VERSION, CRLF) ;
+    fprintf(stdout, "Content-Type: text/plain%s", CRLF) ;
+    fprintf(stdout, CRLF) ;
+
+    fprintf(stdout, "Core software version: %s%s", DVR, CRLF) ;
 
     if (script_ver != "")
-	fprintf( stdout, "Server Script Revision: %s%s", script_ver.c_str(), CRLF ) ;
+        fprintf(stdout, "Server Script Revision: %s%s", script_ver.c_str(), CRLF) ;
 
     if (dataset_ver != "")
-	fprintf( stdout,  "Dataset version: %s%s", dataset_ver.c_str(), CRLF ) ;
+        fprintf(stdout,  "Dataset version: %s%s", dataset_ver.c_str(), CRLF) ;
 
-    fflush( stdout ) ;          // Not sure this is needed. jhrg 12/23/05
-    
-    return true;
-}
-
-#ifndef WIN32
-// // 02/13/98 jhrg
-/** This function sends data to the client.
-
-    This function has been superseded by the DODSFilter::send_data() mfunc.
-
-    @deprecated Use DODSFilter's methods instead.
-
-    @brief Send data. (deprecated)
-    @param compression A Boolean value indicating whether the data is
-    to be sent in compressed form or not.  A value of TRUE indicates
-    that the data is to be comressed.
-    @param data_stream A file pointer indicating where to print the
-    data.  This is typically an XDR pointer.
-    @param dds The DODS Dataset Descriptor Structure of the dataset
-    whose data is to be sent.  The <tt>do_data_transfer</tt> uses the
-    <tt>send</tt> method of this DDS to do the sending.
-    @param dataset The (local) name of the dataset whose data is to be
-    sent.
-    @param constraint A constraint expression that may have been sent
-    with the original data request.
-    @return TRUE for success, FALSE otherwise.
-*/
-bool
-do_data_transfer(bool compression, FILE *data_stream, DDS &dds,
-		 const string &dataset, const string &constraint)
-{
-    if (compression) {
-	int childpid;
-	FILE *data_sink = compressor(data_stream, childpid);
-	if (!dds.send(dataset, constraint, data_sink, true)) {
-	    ErrMsgT("Could not send compressed data");
-	    return false;
-	}
-	fclose(data_sink);
-	int pid;
-	while ((pid = waitpid(childpid, 0, 0)) > 0) {
-	    DBG(cerr << "pid: " << pid << endl);
-	}
-    }
-    else {
-	if (!dds.send(dataset, constraint, data_stream, false)) {
-	    ErrMsgT("Could not send data");
-	    return false;
-	}
-    }
+    fflush(stdout) ;            // Not sure this is needed. jhrg 12/23/05
 
     return true;
 }
-#endif
-
 
 /** This function accepts a dataset path name, and searches for a
     matching ancillary data file name with a very specific set of
@@ -192,9 +142,9 @@ do_data_transfer(bool compression, FILE *data_stream, DDS &dds,
 
     For example, If you call this function with a
     dataset name of <tt>/a/data</tt>, an extension of <tt>das</tt>, a
-    directory of 
+    directory of
     <tt>b</tt>, and a filename of <tt>ralph</tt>, the function will
-    look (in order) 
+    look (in order)
     for the following files:
 
     <pre>
@@ -205,62 +155,63 @@ do_data_transfer(bool compression, FILE *data_stream, DDS &dds,
     </pre>
 
     The function will return a string containing the name of the first
-    file in the list that exists, if any. 
+    file in the list that exists, if any.
 
     @note This code now checks for <code>pathname.ext</code> 3/17/99 jhrg
 
-    @brief Find a file with ancillary data.  
+    @brief Find a file with ancillary data.
     @param pathname The input pathname of a dataset.
     @param ext The input extension the desired file is to have.
     @param dir The input directory in which the desired file may be
-    found. 
+    found.
     @param file The input filename the desired file may have.
     @return A string containing the pathname of the file found by
     searching with the given components.  If no file was found, the
     null string is returned.
 */
 string
-find_ancillary_file(string pathname, string ext, string dir, string file)
+find_ancillary_file(const string &pathname, const string &ext,
+                    const string &dir, const string &file)
 {
     string::size_type slash = pathname.rfind('/') + 1;
     string directory = pathname.substr(0, slash);
     string filename = pathname.substr(slash);
-    string basename = pathname.substr(slash, pathname.rfind('.')-slash);
+    string basename = pathname.substr(slash, pathname.rfind('.') - slash);
 
-    DBG(cerr << "find ancillary file params: " << pathname << ", " << ext 
-	<< ", " << dir << ", " << file << endl);
+    DBG(cerr << "find ancillary file params: " << pathname << ", " << ext
+        << ", " << dir << ", " << file << endl);
     DBG(cerr << "find ancillary file comp: " << directory << ", " << filename
-	<< ", " << basename << endl);
+        << ", " << basename << endl);
 
     string dot_ext = "." + ext;
 
     string name = directory + basename + dot_ext;
     if (access(name.c_str(), F_OK) == 0)
-	return name;
+        return name;
 
     name = pathname + dot_ext;
     if (access(name.c_str(), F_OK) == 0)
-	return name;
+        return name;
 
     name = directory + ext;
     if (access(name.c_str(), F_OK) == 0)
-	return name;
+        return name;
 
     name = dir + basename + dot_ext;
     if (access(name.c_str(), F_OK) == 0)
-	return name;
+        return name;
 
     name = directory + file + dot_ext;
     if (access(name.c_str(), F_OK) == 0)
-	return name;
+        return name;
 
     name = dir + file + dot_ext;
     if (access(name.c_str(), F_OK) == 0)
-	return name;
+        return name;
 
     name = dir + ext;
     if (access(name.c_str(), F_OK) == 0)
-	return name;
+        return name;
 
     return "";
 }
@@ -270,7 +221,7 @@ find_ancillary_file(string pathname, string ext, string dir, string file)
 // is a member. Assume that groups follow a simple naming convention where
 // files use either leading or trailing digits and a common basename to name
 // group memebrs. For example, 00stuff.hdf, 01stuff.hdf, 02stuff.hdf, ..., is
-// a group and is has `stuff' as its basename. 
+// a group and is has `stuff' as its basename.
 
 /** Assume that <tt>name</tt> refers to a file that is one of a
     group of files which share a common `base' name and differ only by
@@ -286,14 +237,14 @@ find_ancillary_file(string pathname, string ext, string dir, string file)
     @return The pathname to the group's ancillary file if found, otherwise
     the empty string (""). */
 string
-find_group_ancillary_file(string name, string ext)
+find_group_ancillary_file(const string &name, const string &ext)
 {
     // Given /usr/local/data/stuff.01.nc
     // pathname = /usr/local/data, filename = stuff.01.nc and
     // rootname = stuff.01
     string::size_type slash = name.find_last_of('/');
     string dirname = name.substr(0, slash);
-    string filename = name.substr(slash+1);
+    string filename = name.substr(slash + 1);
     string rootname = filename.substr(0, filename.find_last_of('.'));
 
     // Instead of using regexs, scan the filename for leading and then
@@ -301,40 +252,40 @@ find_group_ancillary_file(string name, string ext)
     string::iterator rootname_iter = rootname.begin();
     string::iterator rootname_end_iter = rootname.end();
     if (isdigit(*rootname_iter)) {
-	while(rootname_iter != rootname_end_iter 
-	      && isdigit(*++rootname_iter))
-	    ;
+        while (rootname_iter != rootname_end_iter
+               && isdigit(*++rootname_iter))
+            ;
 
-	// We want: new_name = dirname + "/" + <base> + ext but without
-	// creating a bunch of temp objects.
-	string new_name = dirname;
-	new_name.append("/");
-	new_name.append(rootname_iter, rootname_end_iter);
-	new_name.append(ext);
-	DBG(cerr << "New Name (iter): " << new_name << endl);
-	if (access(new_name.c_str(), F_OK) == 0) {
-	    return new_name;
-	}
+        // We want: new_name = dirname + "/" + <base> + ext but without
+        // creating a bunch of temp objects.
+        string new_name = dirname;
+        new_name.append("/");
+        new_name.append(rootname_iter, rootname_end_iter);
+        new_name.append(ext);
+        DBG(cerr << "New Name (iter): " << new_name << endl);
+        if (access(new_name.c_str(), F_OK) == 0) {
+            return new_name;
+        }
     }
 
     string::reverse_iterator rootname_riter = rootname.rbegin();
     string::reverse_iterator rootname_end_riter = rootname.rend();
     if (isdigit(*rootname_riter)) {
-	while(rootname_riter != rootname_end_riter 
-	      && isdigit(*++rootname_riter))
-	    ;
-	string new_name = dirname;
-	new_name.append("/");
-	// I used reverse iters to scan rootname backwards. To avoid
-	// reversing the fragement between end_riter and riter, pass append
-	// regular iters obtained using reverse_iterator::base(). See Meyers
-	// p. 123. 1/22/2002 jhrg
-	new_name.append(rootname_end_riter.base(), rootname_riter.base());
-	new_name.append(ext);
-	DBG(cerr << "New Name (riter): " << new_name << endl);
-	if (access(new_name.c_str(), F_OK) == 0) {
-	    return new_name;
-	}
+        while (rootname_riter != rootname_end_riter
+               && isdigit(*++rootname_riter))
+            ;
+        string new_name = dirname;
+        new_name.append("/");
+        // I used reverse iters to scan rootname backwards. To avoid
+        // reversing the fragement between end_riter and riter, pass append
+        // regular iters obtained using reverse_iterator::base(). See Meyers
+        // p. 123. 1/22/2002 jhrg
+        new_name.append(rootname_end_riter.base(), rootname_riter.base());
+        new_name.append(ext);
+        DBG(cerr << "New Name (riter): " << new_name << endl);
+        if (access(new_name.c_str(), F_OK) == 0) {
+            return new_name;
+        }
     }
 
     // If we're here either the file does not begin with leading digits or a
@@ -352,33 +303,33 @@ find_group_ancillary_file(string name, string ext)
     @brief Logs an error message.
     @return void
 */
-void 
+void
 ErrMsgT(const string &Msgt)
 {
     time_t TimBin;
     char TimStr[TimLen];
 
-    if (time(&TimBin) == (time_t)-1)
-	strcpy(TimStr, "time() error           ");
+    if (time(&TimBin) == (time_t) - 1)
+        strcpy(TimStr, "time() error           ");
     else {
-	strcpy(TimStr, ctime(&TimBin));
-	TimStr[TimLen - 2] = '\0'; // overwrite the \n 
+        strcpy(TimStr, ctime(&TimBin));
+        TimStr[TimLen - 2] = '\0'; // overwrite the \n
     }
 
     const char *host_or_addr = getenv("REMOTE_HOST") ? getenv("REMOTE_HOST") :
-	getenv("REMOTE_ADDR") ? getenv("REMOTE_ADDR") : "local (a non-CGI run)"; 
-    const char *script = getenv("SCRIPT_NAME") ? getenv("SCRIPT_NAME") : 
-	"OPeNDAP server"; 
+                               getenv("REMOTE_ADDR") ? getenv("REMOTE_ADDR") : "local (a non-CGI run)";
+    const char *script = getenv("SCRIPT_NAME") ? getenv("SCRIPT_NAME") :
+                         "OPeNDAP server";
 
-    cerr << "[" << TimStr << "] CGI: " << script << " failed for " 
-	 << host_or_addr << ": "<< Msgt << endl;
+    cerr << "[" << TimStr << "] CGI: " << script << " failed for "
+    << host_or_addr << ": " << Msgt << endl;
 }
 
 // Given a pathname, return just the filename component with any extension
 // removed. The new string resides in newly allocated memory; the caller must
 // delete it when done using the filename.
 // Originally from the netcdf distribution (ver 2.3.2).
-// 
+//
 // *** Change to string class argument and return type. jhrg
 // *** Changed so it also removes the#path#of#the#file# from decompressed
 //     files.  rph.
@@ -391,7 +342,7 @@ ErrMsgT(const string &Msgt)
 
     @brief Returns the filename portion of a pathname.
     @param path A C-style simple string containing a pathname to be
-    parsed. 
+    parsed.
     @return A C-style simple string containing the filename component
     of the given pathname.
 */
@@ -399,13 +350,13 @@ string
 name_path(const string &path)
 {
     if (path == "")
-	return string("");
+        return string("");
 
     string::size_type delim = path.find_last_of(FILE_DELIMITER);
     string::size_type pound = path.find_last_of("#");
     string new_path;
-    
-    if(pound != string::npos)
+
+    if (pound != string::npos)
         new_path = path.substr(pound + 1);
     else
         new_path = path.substr(delim + 1);
@@ -445,9 +396,13 @@ name_path(const string &path)
 //                   / ( ("+" / "-") 4DIGIT )        ; Local differential
 //                                                   ;  hours+min. (HHMM)
 
-static const char *days[]={"Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat"};
-static const char *months[]={"Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", 
-			"Aug", "Sep", "Oct", "Nov", "Dec"};
+static const char *days[] =
+    {"Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat"
+    };
+static const char *months[] =
+    {"Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul",
+     "Aug", "Sep", "Oct", "Nov", "Dec"
+    };
 
 /** Given a constant pointer to a <tt>time_t</tt>, return a RFC
     822/1123 style date.
@@ -463,33 +418,32 @@ rfc822_date(const time_t t)
     struct tm *stm = gmtime(&t);
     char d[256];
 
-    sprintf(d, "%s, %02d %s %4d %02d:%02d:%02d GMT", days[stm->tm_wday], 
-	    stm->tm_mday, months[stm->tm_mon], 
+    sprintf(d, "%s, %02d %s %4d %02d:%02d:%02d GMT", days[stm->tm_wday],
+            stm->tm_mday, months[stm->tm_mon],
 #if 0
-	    // On Solaris 2.7 this tm_year is years since 1900. 3/17/2000
-	    // jhrg
-	    stm->tm_year < 100 ? 1900 + stm->tm_year : stm->tm_year, 
+            // On Solaris 2.7 this tm_year is years since 1900. 3/17/2000
+            // jhrg
+            stm->tm_year < 100 ? 1900 + stm->tm_year : stm->tm_year,
 #endif
-	    1900 + stm->tm_year,
-	    stm->tm_hour, stm->tm_min, stm->tm_sec);
+            1900 + stm->tm_year,
+            stm->tm_hour, stm->tm_min, stm->tm_sec);
     return string(d);
 }
 
 /** Get the last modified time. Assume <tt>name</tt> is a file and
     find its last modified time. If <tt>name</tt> is not a file, then
-    return now as the last modified time. 
+    return now as the last modified time.
     @param name The name of a file.
     @return The last modified time or the current time. */
 time_t
-last_modified_time(string name)
+last_modified_time(const string &name)
 {
     struct stat m;
 
-	if (stat(name.c_str(), &m) == 0 && (S_IFREG & m.st_mode))
-
-	return m.st_mtime;
+    if (stat(name.c_str(), &m) == 0 && (S_IFREG & m.st_mode))
+        return m.st_mtime;
     else
-	return time(0);
+        return time(0);
 }
 
 // Send string to set the transfer (mime) type and server version
@@ -502,9 +456,13 @@ last_modified_time(string name)
 // Returns: false if the compression output filter was to be used but could
 // not be started, true otherwise.
 
-static const char *descrip[]={"unknown", "dods_das", "dods_dds", "dods_data",
-			"dods_error", "web_error"};
-static const char *encoding[]={"unknown", "deflate", "x-plain"};
+static const char *descrip[] =
+    {"unknown", "dods_das", "dods_dds", "dods_data",
+     "dods_error", "web_error", "dap4_ddx"
+    };
+static const char *encoding[] =
+    {"unknown", "deflate", "x-plain"
+    };
 
 /** Generate an HTTP 1.0 response header for a text document. This is used
     when returning a serialized DAS or DDS object.
@@ -512,81 +470,97 @@ static const char *encoding[]={"unknown", "deflate", "x-plain"};
     @param out Write the MIME header to this FILE pointer.
     @param type The type of this this response. Defaults to
     application/octet-stream.
-    @param ver The version string; denotes the DAP spec and implementation
+    @param ver The version string; denotes the libdap implementation
     version.
     @param enc How is this response encoded? Can be plain or deflate or the
     x_... versions of those. Default is x_plain.
     @param last_modified The time to use for the Last-Modified header value.
     Default is zero which means use the current time. */
 void
-set_mime_text(FILE *out, ObjectType type, const string &ver, 
-	      EncodingType enc, const time_t last_modified)
+set_mime_text(FILE *out, ObjectType type, const string &ver,
+              EncodingType enc, const time_t last_modified)
 {
-    fprintf( out, "HTTP/1.0 200 OK%s", CRLF ) ;
-    fprintf( out, "XDODS-Server: %s%s", ver.c_str(), CRLF ) ;
-    fprintf( out, "XOPeNDAP-Server: %s%s", ver.c_str(), CRLF ) ;
-    fprintf( out, "XDAP: %s%s", DAP_PROTOCOL_VERSION, CRLF ) ;
+    fprintf(out, "HTTP/1.0 200 OK%s", CRLF) ;
+    if (ver == "") {
+        fprintf(out, "XDODS-Server: %s%s", DVR, CRLF) ;
+        fprintf(out, "XOPeNDAP-Server: %s%s", DVR, CRLF) ;
+    }
+    else {
+        fprintf(out, "XDODS-Server: %s%s", ver.c_str(), CRLF) ;
+        fprintf(out, "XOPeNDAP-Server: %s%s", ver.c_str(), CRLF) ;
+    }
+    fprintf(out, "XDAP: %s%s", DAP_PROTOCOL_VERSION, CRLF) ;
 
     const time_t t = time(0);
-    fprintf( out, "Date: %s%s", rfc822_date(t).c_str(), CRLF ) ;
+    fprintf(out, "Date: %s%s", rfc822_date(t).c_str(), CRLF) ;
 
-    fprintf( out, "Last-Modified: " ) ;
+    fprintf(out, "Last-Modified: ") ;
     if (last_modified > 0)
-	fprintf( out, "%s%s", rfc822_date(last_modified).c_str(), CRLF ) ;
-    else 
-	fprintf( out, "%s%s", rfc822_date(t).c_str(), CRLF ) ;
+        fprintf(out, "%s%s", rfc822_date(last_modified).c_str(), CRLF) ;
+    else
+        fprintf(out, "%s%s", rfc822_date(t).c_str(), CRLF) ;
+
+    if (type == dap4_ddx)
+        fprintf(out, "Content-Type: text/xml%s", CRLF) ;
+    else
+        fprintf(out, "Content-Type: text/plain%s", CRLF) ;
 
-    fprintf( out, "Content-Type: text/plain%s", CRLF ) ;
     // Note that Content-Description is from RFC 2045 (MIME, pt 1), not 2616.
     // jhrg 12/23/05
-    fprintf( out, "Content-Description: %s%s", descrip[type], CRLF ) ;
-    if (type == dods_error)	// don't cache our error responses.
-	fprintf( out, "Cache-Control: no-cache%s", CRLF ) ;
+    fprintf(out, "Content-Description: %s%s", descrip[type], CRLF) ;
+    if (type == dods_error) // don't cache our error responses.
+        fprintf(out, "Cache-Control: no-cache%s", CRLF) ;
     // Don't write a Content-Encoding header for x-plain since that breaks
     // Netscape on NT. jhrg 3/23/97
     if (enc != x_plain)
-	fprintf( out, "Content-Encoding: %s%s", encoding[enc], CRLF ) ;
-    fprintf( out, CRLF ) ;
+        fprintf(out, "Content-Encoding: %s%s", encoding[enc], CRLF) ;
+    fprintf(out, CRLF) ;
 }
 
 /** Generate an HTTP 1.0 response header for a html document.
 
     @param out Write the MIME header to this FILE pointer.
     @param type The type of this this response.
-    @param ver The version string; denotes the DAP spec and implementation
+    @param ver The version string; denotes the libdap implementation
     version.
     @param enc How is this response encoded? Can be plain or deflate or the
     x_... versions of those. Default is x_plain.
     @param last_modified The time to use for the Last-Modified header value.
     Default is zero which means use the current time. */
 void
-set_mime_html(FILE *out, ObjectType type, const string &ver, 
-	      EncodingType enc, const time_t last_modified)
+set_mime_html(FILE *out, ObjectType type, const string &ver,
+              EncodingType enc, const time_t last_modified)
 {
-    fprintf( out, "HTTP/1.0 200 OK%s", CRLF ) ;
-    fprintf( out, "XDODS-Server: %s%s", ver.c_str(), CRLF ) ;
-    fprintf( out, "XOPeNDAP-Server: %s%s", ver.c_str(), CRLF ) ;
-    fprintf( out, "XDAP: %s%s", DAP_PROTOCOL_VERSION, CRLF ) ;
+    fprintf(out, "HTTP/1.0 200 OK%s", CRLF) ;
+    if (ver == "") {
+        fprintf(out, "XDODS-Server: %s%s", DVR, CRLF) ;
+        fprintf(out, "XOPeNDAP-Server: %s%s", DVR, CRLF) ;
+    }
+    else {
+        fprintf(out, "XDODS-Server: %s%s", ver.c_str(), CRLF) ;
+        fprintf(out, "XOPeNDAP-Server: %s%s", ver.c_str(), CRLF) ;
+    }
+    fprintf(out, "XDAP: %s%s", DAP_PROTOCOL_VERSION, CRLF) ;
 
     const time_t t = time(0);
-    fprintf( out, "Date: %s%s", rfc822_date(t).c_str(), CRLF ) ;
+    fprintf(out, "Date: %s%s", rfc822_date(t).c_str(), CRLF) ;
 
-    fprintf( out, "Last-Modified: " ) ;
+    fprintf(out, "Last-Modified: ") ;
     if (last_modified > 0)
-	fprintf( out, "%s%s", rfc822_date(last_modified).c_str(), CRLF ) ;
-    else 
-	fprintf( out, "%s%s", rfc822_date(t).c_str(), CRLF ) ;
+        fprintf(out, "%s%s", rfc822_date(last_modified).c_str(), CRLF) ;
+    else
+        fprintf(out, "%s%s", rfc822_date(t).c_str(), CRLF) ;
 
-    fprintf( out, "Content-type: text/html%s", CRLF ) ;
+    fprintf(out, "Content-type: text/html%s", CRLF) ;
     // See note above about Content-Description header. jhrg 12/23/05
-    fprintf( out, "Content-Description: %s%s", descrip[type], CRLF ) ;
-    if (type == dods_error)	// don't cache our error responses.
-	fprintf( out, "Cache-Control: no-cache%s", CRLF ) ;
+    fprintf(out, "Content-Description: %s%s", descrip[type], CRLF) ;
+    if (type == dods_error) // don't cache our error responses.
+        fprintf(out, "Cache-Control: no-cache%s", CRLF) ;
     // Don't write a Content-Encoding header for x-plain since that breaks
     // Netscape on NT. jhrg 3/23/97
     if (enc != x_plain)
-	fprintf( out, "Content-Encoding: %s%s", encoding[enc], CRLF ) ;
-    fprintf( out, CRLF ) ;
+        fprintf(out, "Content-Encoding: %s%s", encoding[enc], CRLF) ;
+    fprintf(out, CRLF) ;
 }
 
 /** Write an HTTP 1.0 response header for our binary response document (i.e.,
@@ -595,7 +569,7 @@ set_mime_html(FILE *out, ObjectType type, const string &ver,
     @param out Write the MIME header to this FILE pointer.
     @param type The type of this this response. Defaults to
     application/octet-stream.
-    @param ver The version string; denotes the DAP spec and implementation
+    @param ver The version string; denotes the libdap implementation
     version.
     @param enc How is this response encoded? Can be plain or deflate or the
     x_... versions of those. Default is x_plain.
@@ -603,29 +577,35 @@ set_mime_html(FILE *out, ObjectType type, const string &ver,
     Default is zero which means use the current time.
  */
 void
-set_mime_binary(FILE *out, ObjectType type, const string &ver, 
-		EncodingType enc, const time_t last_modified)
+set_mime_binary(FILE *out, ObjectType type, const string &ver,
+                EncodingType enc, const time_t last_modified)
 {
-    fprintf( out, "HTTP/1.0 200 OK%s", CRLF ) ;
-    fprintf( out, "XDODS-Server: %s%s", ver.c_str(), CRLF ) ;
-    fprintf( out, "XOPeNDAP-Server: %s%s", ver.c_str(), CRLF ) ;
-    fprintf( out, "XDAP: %s%s", DAP_PROTOCOL_VERSION, CRLF ) ;
-   
+    fprintf(out, "HTTP/1.0 200 OK%s", CRLF) ;
+    if (ver == "") {
+        fprintf(out, "XDODS-Server: %s%s", DVR, CRLF) ;
+        fprintf(out, "XOPeNDAP-Server: %s%s", DVR, CRLF) ;
+    }
+    else {
+        fprintf(out, "XDODS-Server: %s%s", ver.c_str(), CRLF) ;
+        fprintf(out, "XOPeNDAP-Server: %s%s", ver.c_str(), CRLF) ;
+    }
+    fprintf(out, "XDAP: %s%s", DAP_PROTOCOL_VERSION, CRLF) ;
+
     const time_t t = time(0);
-    fprintf( out, "Date: %s%s", rfc822_date(t).c_str(), CRLF ) ;
+    fprintf(out, "Date: %s%s", rfc822_date(t).c_str(), CRLF) ;
 
-    fprintf( out, "Last-Modified: " ) ;
+    fprintf(out, "Last-Modified: ") ;
     if (last_modified > 0)
-	fprintf( out, "%s%s", rfc822_date(last_modified).c_str(), CRLF ) ;
-    else 
-	fprintf( out, "%s%s", rfc822_date(t).c_str(), CRLF ) ;
+        fprintf(out, "%s%s", rfc822_date(last_modified).c_str(), CRLF) ;
+    else
+        fprintf(out, "%s%s", rfc822_date(t).c_str(), CRLF) ;
 
-    fprintf( out, "Content-Type: application/octet-stream%s", CRLF ) ;
-    fprintf( out, "Content-Description: %s%s", descrip[type], CRLF ) ;
+    fprintf(out, "Content-Type: application/octet-stream%s", CRLF) ;
+    fprintf(out, "Content-Description: %s%s", descrip[type], CRLF) ;
     if (enc != x_plain)
-	fprintf( out, "Content-Encoding: %s%s", encoding[enc], CRLF ) ;
+        fprintf(out, "Content-Encoding: %s%s", encoding[enc], CRLF) ;
 
-    fprintf( out, CRLF ) ;
+    fprintf(out, CRLF) ;
 }
 
 
@@ -635,25 +615,25 @@ set_mime_binary(FILE *out, ObjectType type, const string &ver,
     @param reason Reason string of the HTTP 1.0 reponse header.
     @param version The version string; denotes the DAP spec and implementation
     version. */
-void 
+void
 set_mime_error(FILE *out, int code, const string &reason,
-	       const string &version)
+               const string &version)
 {
-    fprintf( out, "HTTP/1.0 %d %s%s", code, reason.c_str(), CRLF ) ;
+    fprintf(out, "HTTP/1.0 %d %s%s", code, reason.c_str(), CRLF) ;
     if (version == "") {
-      fprintf( out, "XDODS-Server: %s%s", DVR, CRLF ) ;
-      fprintf( out, "XOPeNDAP-Server: %s%s", DVR, CRLF ) ;
+        fprintf(out, "XDODS-Server: %s%s", DVR, CRLF) ;
+        fprintf(out, "XOPeNDAP-Server: %s%s", DVR, CRLF) ;
     }
     else {
-      fprintf( out, "XDODS-Server: %s%s", version.c_str(), CRLF ) ;
-      fprintf( out, "XOPeNDAP-Server: %s%s", version.c_str(), CRLF ) ;
+        fprintf(out, "XDODS-Server: %s%s", version.c_str(), CRLF) ;
+        fprintf(out, "XOPeNDAP-Server: %s%s", version.c_str(), CRLF) ;
     }
-    fprintf( out, "XDAP: %s%s", DAP_PROTOCOL_VERSION, CRLF ) ;
+    fprintf(out, "XDAP: %s%s", DAP_PROTOCOL_VERSION, CRLF) ;
 
     const time_t t = time(0);
-    fprintf( out, "Date: %s%s", rfc822_date(t).c_str(), CRLF ) ;
-    fprintf( out, "Cache-Control: no-cache%s", CRLF ) ;
-    fprintf( out, CRLF ) ;
+    fprintf(out, "Date: %s%s", rfc822_date(t).c_str(), CRLF) ;
+    fprintf(out, "Cache-Control: no-cache%s", CRLF) ;
+    fprintf(out, CRLF) ;
 }
 
 
@@ -663,13 +643,13 @@ set_mime_error(FILE *out, int code, const string &reason,
 
     @brief Send a `Not Modified' response.
     @param out Write the response to this FILE pointer. */
-void 
+void
 set_mime_not_modified(FILE *out)
 {
-    fprintf( out, "HTTP/1.0 304 NOT MODIFIED%s", CRLF ) ;
+    fprintf(out, "HTTP/1.0 304 NOT MODIFIED%s", CRLF) ;
     const time_t t = time(0);
-    fprintf( out, "Date: %s%s", rfc822_date(t).c_str(), CRLF ) ;
-    fprintf( out, CRLF ) ;
+    fprintf(out, "Date: %s%s", rfc822_date(t).c_str(), CRLF) ;
+    fprintf(out, CRLF) ;
 }
 
 /** Look for the override file by taking the dataset name and
@@ -685,14 +665,14 @@ found_override(string name, string &doc)
 {
     ifstream ifs((name + ".ovr").c_str());
     if (!ifs)
-	return false;
+        return false;
 
     char tmp[256];
     doc = "";
     while (!ifs.eof()) {
-	ifs.getline(tmp, 255);
-	strcat(tmp, "\n");
-	doc += tmp;
+        ifs.getline(tmp, 255);
+        strcat(tmp, "\n");
+        doc += tmp;
     }
 
     return true;
@@ -711,13 +691,13 @@ remove_mime_header(FILE *in)
 {
     char tmp[256];
     while (!feof(in)) {
-	fgets(tmp, 255, in);
-	if (strncmp(&tmp[0], CRLF, 2) == 0)
-	    return true;
+        fgets(tmp, 255, in);
+        if (strncmp(&tmp[0], CRLF, 2) == 0)
+            return true;
     }
 
     return false;
-}    
+}
 
 
 /** Look in the CGI directory (given by \c cgi) for a per-cgi HTML* file.
@@ -737,7 +717,7 @@ remove_mime_header(FILE *in)
     tags (my own notation).
 
     @brief Look for the user supplied CGI- and dataset-specific HTML*
-    documents. 
+    documents.
 
     @return A String which contains these two documents catenated. Documents
     that don't exist are treated as `empty'.  */
@@ -750,13 +730,13 @@ get_user_supplied_docs(string name, string cgi)
     ifstream ifs((cgi + ".html").c_str());
 
     if (ifs) {
-	while (!ifs.eof()) {
-	    ifs.getline(tmp, 255);
-	    oss << tmp << "\n";
-	}
-	ifs.close();
-	
-	oss << "<hr>";
+        while (!ifs.eof()) {
+            ifs.getline(tmp, 255);
+            oss << tmp << "\n";
+        }
+        ifs.close();
+
+        oss << "<hr>";
     }
 
     // Problem: This code is run with the CWD as the CGI-BIN directory but
@@ -768,17 +748,17 @@ get_user_supplied_docs(string name, string cgi)
 
     // If name.html cannot be opened, look for basename.html
     if (!ifs) {
-	string new_name = find_group_ancillary_file(name, ".html");
-	if (new_name != "")
-	    ifs.open(new_name.c_str());
+        string new_name = find_group_ancillary_file(name, ".html");
+        if (new_name != "")
+            ifs.open(new_name.c_str());
     }
 
     if (ifs) {
-	while (!ifs.eof()) {
-	    ifs.getline(tmp, 255);
-	    oss << tmp << "\n";
-	}
-	ifs.close();
+        while (!ifs.eof()) {
+            ifs.getline(tmp, 255);
+            oss << tmp << "\n";
+        }
+        ifs.close();
     }
 
     return oss.str();
