@@ -42,6 +42,7 @@
 class TestArray: public Array, public TestCommon {
     bool d_series_values;
     void _duplicate(const TestArray &ts);
+    int m_offset(int y, Dim_iter Y, int x);
 
 public:
     TestArray(const string &n = "", BaseType *v = 0);
@@ -54,6 +55,10 @@ public:
     virtual BaseType *ptr_duplicate();
 
     virtual bool read(const string &dataset);
+    virtual bool name_is_special();
+    virtual void build_special_values();    
+    virtual void constrained_matrix(const string &dataset, char *constrained_array);
+    
     void set_series_values(bool);
     bool get_series_values() { return d_series_values; }
 };
